

package client;

import service.*;

import java.io.*;

import java.rmi.*; 

import net.jini.discovery.LookupDiscovery;
import net.jini.discovery.LookupDiscoveryManager;
import net.jini.core.lookup.ServiceTemplate;
import net.jini.core.lookup.ServiceItem;
import net.jini.lookup.ServiceDiscoveryManager;
import net.jini.lookup.ServiceDiscoveryListener;
import net.jini.lookup.ServiceDiscoveryEvent;
import net.jini.lease.LeaseRenewalManager;

import net.jini.lookup.LookupCache;

public class Client implements  ServiceDiscoveryListener {

    private LookupCache cache = null;

    public static void main(String[] args) {
	new Client();
    }

    public Client() {
	System.setSecurityManager(new RMISecurityManager());

	// Build a cache of all discovered echo services and monitor changes
	ServiceDiscoveryManager serviceMgr = null;
	Class [] classes = new Class[] {Echo.class};
	ServiceTemplate template = new ServiceTemplate(null, classes, 
							   null);	
	try {
            LookupDiscoveryManager mgr =
                new LookupDiscoveryManager(LookupDiscovery.ALL_GROUPS,
                                           null,  // unicast locators
                                           null); // DiscoveryListener
            serviceMgr = new ServiceDiscoveryManager(mgr, 
						    new LeaseRenewalManager());
        } catch(Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
	
        try {
            cache = serviceMgr.createLookupCache(template, 
                                                null,  // no filter
                                                this); // listener
        } catch(Exception e) {
            e.printStackTrace();
            System.exit(1);
        }

	loop();
    }

    public void loop() {
	for (int n = 0; n < 10; n++) {
	    System.out.println("Sending: hello " + n);
	    echoToAll("hello " +  n);
	    try {
		Thread.currentThread().sleep(1000);
	    } catch(Exception e) {
	    }
	}
	System.exit(0);
    }

    private void echoToAll(String txt) {
	ServiceItem[] items = cache.lookup(null, Integer.MAX_VALUE);
	for (int n  = 0; n < items.length; n++) {
	    Echo svc = (Echo) items[n].service;
	    echoToOne(svc, txt);
	}
    }

    private void echoToOne(Echo svc, String txt) {
	String reply = null;
	try {
	    reply = svc.echo(txt);
	    System.out.println(reply);
	} catch(RemoteException e) {
	    System.out.println(e);
	}
    }

    public void serviceAdded(ServiceDiscoveryEvent evt) {
	// evt.getPreEventServiceItem() == null
	ServiceItem postItem = evt.getPostEventServiceItem();
	System.out.println("Service appeared: " +
			   postItem.service.getClass().toString());
    }

    public void serviceChanged(ServiceDiscoveryEvent evt) {
	ServiceItem preItem = evt.getPostEventServiceItem();
	ServiceItem postItem = evt.getPreEventServiceItem() ;
	System.out.println("Service changed: " +
			   postItem.service.getClass().toString());
    }
    public void serviceRemoved(ServiceDiscoveryEvent evt) {
	// evt.getPostEventServiceItem() == null
	ServiceItem preItem = evt.getPreEventServiceItem();
	System.out.println("Service disappeared: " +
			   preItem.service.getClass().toString());
    }

}

